/*
   Author: Olaru Mihai Alexandru
   Grupa: 335CA
*/
#define F_CPU 16000000

#include <avr/io.h>
#include <stdio.h>
#include <stdlib.h>
#include <util/delay.h>
#include <string.h>

#include "lcd/nokia_5110.h"
#include "lcd/bmp_pixel.h"

#define MOTOR_DELAY 432

#define PRICE_1 25
#define PRICE_2 30
#define PRICE_3 25
#define PRICE_4 20

#define COIN_VALUE 5

int credit = 100;

int x = 20;
int y = 3;

/*
   Debugg using a led
*/
void led_init() {
   DDRD |= (1 << PD7);
   PORTD &= ~(1 << PD7);
}

/*
   Init Motor Pins
*/
void motors_init() {
   // Motor 1
   DDRB |= (1 << PB3);
   DDRB |= (1 << PB4);

   // Motor 2
   DDRD |= (1 << PD4);
   DDRD |= (1 << PD6);

   // Motor 3
   DDRD |= (1 << PD5);
   DDRD |= (1 << PD7);

   // Motor 4
   DDRD |= (1 << PD0);
   DDRD |= (1 << PD1);
}

/*
   Buttons Init
*/
void button_init() {
   DDRA &= ~(1 << PA0); //seteaza pin ca intrare
   PORTA |= (1 << PA0); //pull up pt buton 1

   DDRA &= ~(1 << PA1); //seteaza pin ca intrare
   PORTA |= (1 << PA1); //pull up pt buton 2

   DDRA &= ~(1 << PA2); //seteaza pin ca intrare
   PORTA |= (1 << PA2); //pull up pt buton 3

   DDRA &= ~(1 << PA3); //seteaza pin ca intrare
   PORTA |= (1 << PA3); //pull up pt buton 4
}


/*
   Motor1 e conectat la PB3 si PB4
*/
void run_motor1_ccw() {
   PORTB |= (1 << PB3);
}

void motor1_complete_rotation_ccw() {
   run_motor1_ccw();
   _delay_ms(MOTOR_DELAY);
   PORTB &= ~(1 << PB3);
}

/*
   Motor2 e conectat la PD4 si PD6
*/
void run_motor2_ccw() {
   PORTD |= (1 << PD4);
}

void motor2_complete_rotation_ccw() {
   run_motor2_ccw();
   _delay_ms(MOTOR_DELAY);
   PORTD &= ~(1 << PD4);
}

/*
   Motor3 e conectat la PD5 si PD7
*/
void run_motor3_ccw() {
   PORTD |= (1 << PD5);
}

void motor3_complete_rotation_ccw() {
   run_motor3_ccw();
   _delay_ms(MOTOR_DELAY);
   PORTD &= ~(1 << PD5);
}

/*
   Motor4 e conectat la PD0 si PD1
*/
void run_motor4_ccw() {
   PORTD |= (1 << PD0);
}

void motor4_complete_rotation_ccw() {
   run_motor4_ccw();
   _delay_ms(MOTOR_DELAY);
   PORTD &= ~(1 << PD0);
}

void ADC_init(void)
{
	/* initalizare ADC */
	ADMUX = (1 << REFS0);
	ADCSRA = (1 << ADEN) | (7 << ADPS0);
}

int ADC_get(uint8_t channel)
{
	/* portneste adc
	 * asteapta conversia sa se termine si returneaza valoarea
	 */
	ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

	ADCSRA |= (1 << ADSC);
	while(ADCSRA & (1 << ADSC));

   return ADC;
}



/*
   Intoarce numarul butonului apasat
*/
int get_button_pressed() {
   if ((PINA & (1 << PA0)) == 0)
      return 1;
   else if ((PINA & (1 << PA1)) == 0)
      return 2;
   else if ((PINA & (1 << PA2)) == 0)
      return 3;
   else if ((PINA & (1 << PA3)) == 0)
      return 4;   

   return 0;
}

/*
   Se afiseaza daca creditul nu este suficient pentru
   a cumpara un anume produs
*/
void credit_insuficient() {
   LCD_write_english_string(10, y, "INSUFICIENT");
   _delay_ms(3000);
   LCD_write_english_string(0, y, "                        ");
}

/*
   In functie de butonul apasat si de creditul disponibil,
   invarte motorul corespunzator butonului
*/
void button_action(int nr_button) {
   switch (nr_button)
   {
   case 1:
      if (credit >= PRICE_1) {
         credit -= PRICE_1;
         motor1_complete_rotation_ccw();
         _delay_ms(1000);
      } else {
        credit_insuficient();
      }
      break;
   case 2:
      if (credit >= PRICE_2) {
         credit -= PRICE_2;
        motor2_complete_rotation_ccw();
        _delay_ms(1000);
      } else {
        credit_insuficient();
      }
      break;

   case 3:
      if (credit >= PRICE_3) {
         credit -= PRICE_3;
         motor3_complete_rotation_ccw();
         _delay_ms(1000);
      } else {
        credit_insuficient();
      }
      break;   
   case 4:
      if (credit >= PRICE_4) {
         credit -= PRICE_4;
         motor3_complete_rotation_ccw();
         _delay_ms(1000);
      } else {
        credit_insuficient();
      }
      break;      
    default:
      break;
   }  
}

/*
   Afiseaza creditul ramas
   am incercat in mai multe feluri, dar presat de timp
   asa a mers cel mai bine avand in vedere functiile 
   penru LCD. Daca gasesti o metoda mai 
   eleganta pls lemme know.
*/
void show_credit () {
   switch (credit)
   {
   case 100:
      LCD_write_english_string(x, y, "10");
      break;
   case 95:
      LCD_write_english_string(x, y, "9,5");
      break;
   case 90:
      LCD_write_english_string(x, y, "9  ");
      break;
   case 85:
      LCD_write_english_string(x, y, "8,5");
      break;
   case 80:
      LCD_write_english_string(x, y, "8  ");
      break;
   case 75:
      LCD_write_english_string(x, y, "7,5");
      break;
   case 70:
      LCD_write_english_string(x, y, "7  ");
      break;
   case 65:
      LCD_write_english_string(x, y, "6,5");
      break;
   case 60:
      LCD_write_english_string(x, y, "6  ");
      break;
   case 55:
      LCD_write_english_string(x, y, "5,5");
      break;
   case 50:
      LCD_write_english_string(x, y, "5  ");
      break;
   case 45:
      LCD_write_english_string(x, y, "4,5");
      break;
   case 40:
      LCD_write_english_string(x, y, "4 ");
      break;
   case 35:
      LCD_write_english_string(x, y, "3,5");
      break;
   case 30:
      LCD_write_english_string(x, y, "3  ");
      break;
   case 25:
      LCD_write_english_string(x, y, "2,5");
      break;
   case 20:
      LCD_write_english_string(x, y, "2  ");
      break;
   case 15:
      LCD_write_english_string(x, y, "1,5");
      break;
   case 10:
      LCD_write_english_string(x, y, "1");
      break;
   case 5:
      LCD_write_english_string(x, y, "0,5 ");
      break;

   default:
      LCD_write_english_string(x, y, "0  ");
      break;
   }

   LCD_write_english_string(44, 3, "LEI");
}

int main() {
   ADC_init();
   button_init();
   motors_init();

   LCD_init();       //LCD initialization
	LCD_clear();

   DDRA &= ~(1 << PA7); //infrarosu init

   while (1) {

      LCD_write_english_string(12, 1, "BUN VENIT!");
      LCD_write_english_string(21, 2, "CREDIT:");
      show_credit();

      if (ADC_get(7) > 1000) { // verifica daca s-a introdus o moneda
         credit += COIN_VALUE;
         _delay_ms(1000);
      }
      
      int button_nr = get_button_pressed();

      button_action(button_nr);
   }

   return 0;

}
